from abc import ABC, abstractmethod
from typing import Optional

from cpl_query.extension import List

from bot_data.model.auto_role import AutoRole
from bot_data.model.auto_role_rule import AutoRoleRule
from bot_data.model.server import Server


class AutoRoleRepositoryABC(ABC):

    @abstractmethod
    def __init__(self): pass

    @abstractmethod
    def get_auto_roles(self) -> List[AutoRole]: pass

    @abstractmethod
    def get_auto_role_by_id(self, id: int) -> AutoRole: pass

    @abstractmethod
    def get_auto_roles_by_server_id(self, id: int) -> AutoRole: pass

    @abstractmethod
    def find_auto_roles_by_server_id(self, id: int) -> Optional[AutoRole]: pass

    @abstractmethod
    def get_auto_roles_by_message_id(self, id: int) -> AutoRole: pass

    @abstractmethod
    def find_auto_roles_by_message_id(self, id: int) -> Optional[AutoRole]: pass

    @abstractmethod
    def add_auto_role(self, server: AutoRole): pass

    @abstractmethod
    def update_auto_role(self, server: AutoRole): pass

    @abstractmethod
    def delete_auto_role(self, server: AutoRole): pass

    @abstractmethod
    def get_auto_role_rules(self) -> List[AutoRoleRule]: pass

    @abstractmethod
    def get_auto_role_rule_by_id(self, id: int) -> AutoRoleRule: pass

    @abstractmethod
    def get_auto_role_rules_by_auto_role_id(self, id: int) -> List[AutoRoleRule]: pass

    @abstractmethod
    def add_auto_role_rule(self, server: AutoRoleRule): pass

    @abstractmethod
    def update_auto_role_rule(self, server: AutoRoleRule): pass

    @abstractmethod
    def delete_auto_role_rule(self, server: AutoRoleRule): pass
